

#variable started 0

if {![info exists started]} {variable started 0}

if {$started != 0} {

     variable JS ""
     variable LogFlag

     if {$LogFlag == 1} {InsertLog "Last Updated: [clock format [clock seconds] -format "%B %d, %Y %H:%M:%S"]"}

    do_click
}

if {$started == 0} {           ;#  first time load code and define local variables
    set started 1


    variable LogText ""
    variable LogFlag 0
    variable LogCount 0

    variable ShowLogFlag 0

    variable JS ""

    variable A16Base 0x0000
    variable A24Base 0x000000
    variable A32Base 0x00000000

    variable AccessMode A16U
    variable WidthMode D16

    variable Command ""

#    bus access procedures

    proc rd {address} {
        variable A16Base
        variable A24Base
        variable A32Base
        variable AccessMode
        variable WidthMode
        variable LogFlag
        variable ShowLogFlag
        variable JS

       set Base [set [string range $AccessMode 0 2]Base]

        switch $AccessMode {
            A16S  {set AddrMod 0x2d}
            A16 -
            A16U  {set AddrMod 0x29}
            A24 -
            A24UD {set AddrMod 0x39}
            A24UP {set AddrMod 0x3a}
            A24SD {set AddrMod 0x3d}
            A24SP {set AddrMod 0x3e}
            A32 -
            A32UD {set AddrMod 0x09}
            A32UP {set AddrMod 0x0a}
            A32SD {set AddrMod 0x0d}
            A32SP {set AddrMod 0x0e}
            default {
                InsertLog "Address Modifier is invalid"
                return
            }
        }

        switch $WidthMode {
            D8   {set DWidth 8}
            D16  {set DWidth 16}
            D32  {set DWidth 32}
            default {
                InsertLog "Data Width is invalid"
                return
            }
        }

        set rc [rdVME -a [expr $Base + $address] -w $DWidth -m $AddrMod]

         if {$ShowLogFlag == 1} {
             InsertLog "$AccessMode-$WidthMode: rd [format 0x%04x [expr $Base + $address]]: $rc"
         } else {
             append JS "alert(\"$AccessMode-$WidthMode: rd [format 0x%04x [expr $Base + $address]]: $rc\");"
         }

       return [second $rc]
    }

    proc wr {address data} {
        variable A16Base
        variable A24Base
        variable A32Base
        variable AccessMode
        variable WidthMode
        variable LogFlag
        variable ShowLogFlag
        variable JS

       set Base [set [string range $AccessMode 0 2]Base]

       switch $AddressModifier {
            A16S  {set AddrMod 0x2d}
            A16 -
            A16U  {set AddrMod 0x29}
            A24 -
            A24UD {set AddrMod 0x39}
            A24UP {set AddrMod 0x3a}
            A24SD {set AddrMod 0x3d}
            A24SP {set AddrMod 0x3e}
            A32 -
            A32UD {set AddrMod 0x09}
            A32UP {set AddrMod 0x0a}
            A32SD {set AddrMod 0x0d}
            A32SP {set AddrMod 0x0e}
            default {
                return [rpcvar::rpcvar string "0x10004 Address Modifier is invalid"]
            }
        }

        switch $DataWidth {
            D8   {set DWidth 8}
            D16  {set DWidth 16}
            D32  {set DWidth 32}
            default {
                return [rpcvar::rpcvar string "0x10004 Data Width is invalid"]
            }
        }

        set rc [wtVME -a [expr $Base + $address] -w $DWidth -m $AddrMod -d $data]

         if {$ShowLogFlag == 1} {
              InsertLog "$AccessMode-$WidthMode: wr [format 0x%04x [expr $Base + $address]]: $rc"
         } else {
             append JS "alert(\"$AccessMode-$WidthMode: wr [format 0x%04x [expr $Base + $address]]: $rc\");"
         }

       return [second $rc]
    }

   proc show_variables {} {
        variable A16Base
        variable A24Base
        variable A32Base
        variable AccessMode
        variable WidthMode
        variable ShowLogFlag
        variable LogFlag
        variable JS

         if {$ShowLogFlag == 1} {
             EmptyLog
             InsertLog "A16Base=$A16Base; A24Base=$A24Base; A32Base=$A32Base"
             InsertLog "AccessMode=$AccessMode; WidthMode=$WidthMode"
             InsertLog "LogFlag=$LogFlag; ShowLogFlag=$ShowLogFlag"
         }  else {
             append JS "alert(\"A16Base=$A16Base; A24Base=$A24Base; A32Base=$A32Base; AccessMode=$AccessMode; WidthMode=$WidthMode; LogFlag=$LogFlag; ShowLogFlag=$ShowLogFlag"
             append JS "\");"
         }
   }

   proc do_parameters {} {
        variable cgidata
        variable LogFlag

        for {set i 2} {$i < [llength $cgidata]} {incr i} {

            set name [lindex $cgidata $i]
            incr i
            set value [lindex $cgidata $i]

            switch -glob $name {

               Command -
               AccessMode -
               WidthMode -
               A16Base  -
               A24Base  -
               A32Base  {if {$LogFlag == 1} {InsertLog "$name = $value"};variable $name; set $name $value}
               default {}
            }
        }
   }

     proc do_click {} {
       variable cgidata
       variable started
       variable LogFlag
       variable ShowLogFlag
       variable Command

       set cgidata [ncgi::nvlist]

       if {$LogFlag == 1} {InsertLog "cgidata= ([llength $cgidata]) $cgidata"}

       if {[llength $cgidata] == 0} then {return}

       if {[llength $cgidata] < 2} then {
           InsertLog "CGI data is too short: $cgidata"
       } else {
           if {[lindex $cgidata 0] != "Widget"} then {
               InsertLog "Invalid CGI data: $cgidata"
           } else {
               set w [lindex $cgidata 1]

               do_parameters

               switch -glob $w {

                   RESET    {set started 0}
                   CLEAR    {EmptyLog}
                   ELOG     {PrintLog}
                   RELOAD   {}
                   VARIABLES {show_variables}

                   HIDE     {set ShowLogFlag 0}
                   SHOW     {set ShowLogFlag 1}
                   LOGON    {set LogFlag 1}
                   LOGOFF   {set LogFlag 0}

                   COMMAND  {
                       set z [catch {eval $Command} m]
                       if {$z == 0} {
                          InsertLog "$m"
                       } else {
                          InsertLog "command error: rc=$z: $m"
                       }
                   }

                   MainLog {}
                   undefined {}
                   default {InsertLog "unexpected CGI data:  $cgidata"}
               }
           }
       }
    }

}   ;# end of if {$started == 0}

